package javax.management;

import java.security.PrivilegedAction;
import java.util.WeakHashMap;
import java.security.AccessController;
import javax.management.openmbean.OpenMBeanAttributeInfoSupport;
import javax.management.openmbean.OpenMBeanAttributeInfo;
import javax.management.openmbean.OpenMBeanOperationInfoSupport;
import javax.management.openmbean.OpenMBeanOperationInfo;
import javax.management.openmbean.OpenMBeanConstructorInfoSupport;
import javax.management.openmbean.OpenMBeanConstructorInfo;
import javax.management.openmbean.OpenMBeanParameterInfoSupport;
import javax.management.openmbean.OpenMBeanParameterInfo;
import java.util.HashMap;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import com.sun.jmx.mbeanserver.MXBeanSupport;
import com.sun.jmx.mbeanserver.StandardMBeanSupport;
import com.sun.jmx.mbeanserver.Introspector;
import com.sun.jmx.mbeanserver.Util;
import java.util.Map;
import com.sun.jmx.mbeanserver.MBeanSupport;
import com.sun.jmx.mbeanserver.DescriptorCache;

public class StandardMBean implements DynamicMBean, MBeanRegistration
{
    private static final DescriptorCache descriptors;
    private volatile MBeanSupport<?> mbean;
    private volatile MBeanInfo cachedMBeanInfo;
    private static final Map<Class<?>, Boolean> mbeanInfoSafeMap;
    
    private <T> void construct(T cast, Class<T> clazz, boolean b, boolean b2) throws NotCompliantMBeanException {
        if (cast == null) {
            if (b) {
                cast = Util.cast(this);
            }
            else {
                throw new IllegalArgumentException("implementation is null");
            }
        }
        if (b2) {
            if (clazz == null) {
                clazz = (Class)Util.cast(Introspector.getMXBeanInterface(cast.getClass()));
            }
            this.mbean = new MXBeanSupport(cast, clazz);
        }
        else {
            if (clazz == null) {
                clazz = (Class)Util.cast(Introspector.getStandardMBeanInterface(cast.getClass()));
            }
            this.mbean = new StandardMBeanSupport(cast, clazz);
        }
    }
    
    public <T> StandardMBean(T t, Class<T> clazz) throws NotCompliantMBeanException {
        this.construct(t, clazz, false, false);
    }
    
    protected StandardMBean(Class<?> clazz) throws NotCompliantMBeanException {
        this.construct(null, clazz, true, false);
    }
    
    public <T> StandardMBean(T t, Class<T> clazz, boolean b) {
        try {
            this.construct(t, clazz, false, b);
        }
        catch (NotCompliantMBeanException cause) {
            throw new IllegalArgumentException(cause);
        }
    }
    
    protected StandardMBean(Class<?> clazz, boolean b) {
        try {
            this.construct(null, clazz, true, b);
        }
        catch (NotCompliantMBeanException cause) {
            throw new IllegalArgumentException(cause);
        }
    }
    
    public void setImplementation(Object o) throws NotCompliantMBeanException {
        if (o == null) {
            throw new IllegalArgumentException("implementation is null");
        }
        if (this.isMXBean()) {
            this.mbean = new MXBeanSupport(o, (Class)Util.cast(this.getMBeanInterface()));
        }
        else {
            this.mbean = new StandardMBeanSupport(o, (Class)Util.cast(this.getMBeanInterface()));
        }
        return;
    }
    
    public Object getImplementation() {
        return this.mbean.getResource();
    }
    
    public final Class<?> getMBeanInterface() {
        return this.mbean.getMBeanInterface();
    }
    
    public Class<?> getImplementationClass() {
        return this.mbean.getResource().getClass();
    }
    
    @Override
    public Object getAttribute(String attribute) throws AttributeNotFoundException, MBeanException, ReflectionException {
        return this.mbean.getAttribute(attribute);
    }
    
    @Override
    public void setAttribute(Attribute attribute) throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        this.mbean.setAttribute(attribute);
    }
    
    @Override
    public AttributeList getAttributes(String[] attributes) {
        return this.mbean.getAttributes(attributes);
    }
    
    @Override
    public AttributeList setAttributes(AttributeList attributes) {
        return this.mbean.setAttributes(attributes);
    }
    
    @Override
    public Object invoke(String operation, Object[] params, String[] signature) throws MBeanException, ReflectionException {
        return this.mbean.invoke(operation, params, signature);
    }
    
    @Override
    public MBeanInfo getMBeanInfo() {
        try {
            MBeanInfo cachedMBeanInfo = this.getCachedMBeanInfo();
            if (cachedMBeanInfo != null) {
                return cachedMBeanInfo;
            }
        }
        catch (RuntimeException thrown) {
            if (JmxProperties.MISC_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MISC_LOGGER.logp(Level.FINEST, MBeanServerFactory.class.getName(), "getMBeanInfo", "Failed to get cached MBeanInfo", thrown);
            }
        }
        if (JmxProperties.MISC_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MISC_LOGGER.logp(Level.FINER, MBeanServerFactory.class.getName(), "getMBeanInfo", "Building MBeanInfo for " + this.getImplementationClass().getName());
        }
        MBeanSupport<Object> mbean = this.mbean;
        MBeanInfo mBeanInfo = mbean.getMBeanInfo();
        MBeanInfo mBeanInfo2 = new MBeanInfo(this.getClassName(mBeanInfo), this.getDescription(mBeanInfo), this.getAttributes(mBeanInfo), this.getConstructors(mBeanInfo, mbean.getResource()), this.getOperations(mBeanInfo), this.getNotifications(mBeanInfo), this.getDescriptor(mBeanInfo, immutableInfo(this.getClass())));
        try {
            this.cacheMBeanInfo(mBeanInfo2);
        }
        catch (RuntimeException thrown2) {
            if (JmxProperties.MISC_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.MISC_LOGGER.logp(Level.FINEST, MBeanServerFactory.class.getName(), "getMBeanInfo", "Failed to cache MBeanInfo", thrown2);
            }
        }
        return mBeanInfo2;
    }
    
    protected String getClassName(MBeanInfo mBeanInfo) {
        if (mBeanInfo == null) {
            return this.getImplementationClass().getName();
        }
        return mBeanInfo.getClassName();
    }
    
    protected String getDescription(MBeanInfo mBeanInfo) {
        if (mBeanInfo == null) {
            return null;
        }
        return mBeanInfo.getDescription();
    }
    
    protected String getDescription(MBeanFeatureInfo mBeanFeatureInfo) {
        if (mBeanFeatureInfo == null) {
            return null;
        }
        return mBeanFeatureInfo.getDescription();
    }
    
    protected String getDescription(MBeanAttributeInfo mBeanAttributeInfo) {
        return this.getDescription(mBeanAttributeInfo);
    }
    
    protected String getDescription(MBeanConstructorInfo mBeanConstructorInfo) {
        return this.getDescription(mBeanConstructorInfo);
    }
    
    protected String getDescription(MBeanConstructorInfo mBeanConstructorInfo, MBeanParameterInfo mBeanParameterInfo, int n) {
        if (mBeanParameterInfo == null) {
            return null;
        }
        return mBeanParameterInfo.getDescription();
    }
    
    protected String getParameterName(MBeanConstructorInfo mBeanConstructorInfo, MBeanParameterInfo mBeanParameterInfo, int n) {
        if (mBeanParameterInfo == null) {
            return null;
        }
        return mBeanParameterInfo.getName();
    }
    
    protected String getDescription(MBeanOperationInfo mBeanOperationInfo) {
        return this.getDescription(mBeanOperationInfo);
    }
    
    protected int getImpact(MBeanOperationInfo mBeanOperationInfo) {
        if (mBeanOperationInfo == null) {
            return 3;
        }
        return mBeanOperationInfo.getImpact();
    }
    
    protected String getParameterName(MBeanOperationInfo mBeanOperationInfo, MBeanParameterInfo mBeanParameterInfo, int n) {
        if (mBeanParameterInfo == null) {
            return null;
        }
        return mBeanParameterInfo.getName();
    }
    
    protected String getDescription(MBeanOperationInfo mBeanOperationInfo, MBeanParameterInfo mBeanParameterInfo, int n) {
        if (mBeanParameterInfo == null) {
            return null;
        }
        return mBeanParameterInfo.getDescription();
    }
    
    protected MBeanConstructorInfo[] getConstructors(MBeanConstructorInfo[] array, Object o) {
        if (array == null) {
            return null;
        }
        if (o != null && o != this) {
            return null;
        }
        return array;
    }
    
    MBeanNotificationInfo[] getNotifications(MBeanInfo mBeanInfo) {
        return null;
    }
    
    Descriptor getDescriptor(MBeanInfo mBeanInfo, boolean b) {
        if (mBeanInfo == null || mBeanInfo.getDescriptor() == null || mBeanInfo.getDescriptor().getFieldNames().length == 0) {
            return StandardMBean.descriptors.get(new ImmutableDescriptor(new String[] { "interfaceClassName=" + this.getMBeanInterface().getName(), "immutableInfo=" + b }));
        }
        Descriptor descriptor = mBeanInfo.getDescriptor();
        HashMap<String, String> hashMap = new HashMap();
        for (String s : descriptor.getFieldNames()) {
            if (s.equals("immutableInfo")) {
                hashMap.put(s, Boolean.toString(b));
            }
            else {
                hashMap.put(s, descriptor.getFieldValue(s));
            }
        }
        return new ImmutableDescriptor(hashMap);
    }
    
    protected MBeanInfo getCachedMBeanInfo() {
        return this.cachedMBeanInfo;
    }
    
    protected void cacheMBeanInfo(MBeanInfo cachedMBeanInfo) {
        this.cachedMBeanInfo = cachedMBeanInfo;
    }
    
    private boolean isMXBean() {
        return this.mbean.isMXBean();
    }
    
    private static <T> boolean identicalArrays(T[] array, T[] array2) {
        if (array == array2) {
            return true;
        }
        if (array == null || array2 == null || array.length != array2.length) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] != array2[i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static <T> boolean equal(T t, T obj) {
        return t == obj || (t != null && obj != null && t.equals(obj));
    }
    
    private static MBeanParameterInfo customize(MBeanParameterInfo mBeanParameterInfo, String name, String description) {
        if (equal(name, mBeanParameterInfo.getName()) && equal(description, mBeanParameterInfo.getDescription())) {
            return mBeanParameterInfo;
        }
        if ((mBeanParameterInfo instanceof OpenMBeanParameterInfo)) {
            return new OpenMBeanParameterInfoSupport(name, description, ((OpenMBeanParameterInfo)mBeanParameterInfo).getOpenType(), mBeanParameterInfo.getDescriptor());
        }
        return new MBeanParameterInfo(name, mBeanParameterInfo.getType(), description, mBeanParameterInfo.getDescriptor());
    }
    
    private static MBeanConstructorInfo customize(MBeanConstructorInfo mBeanConstructorInfo, String description, MBeanParameterInfo[] array) {
        if (equal(description, mBeanConstructorInfo.getDescription()) && identicalArrays(array, mBeanConstructorInfo.getSignature())) {
            return mBeanConstructorInfo;
        }
        if ((mBeanConstructorInfo instanceof OpenMBeanConstructorInfo)) {
            return new OpenMBeanConstructorInfoSupport(mBeanConstructorInfo.getName(), description, paramsToOpenParams(array), mBeanConstructorInfo.getDescriptor());
        }
        return new MBeanConstructorInfo(mBeanConstructorInfo.getName(), description, array, mBeanConstructorInfo.getDescriptor());
    }
    
    private static MBeanOperationInfo customize(MBeanOperationInfo mBeanOperationInfo, String description, MBeanParameterInfo[] array, int impact) {
        if (equal(description, mBeanOperationInfo.getDescription()) && identicalArrays(array, mBeanOperationInfo.getSignature()) && impact == mBeanOperationInfo.getImpact()) {
            return mBeanOperationInfo;
        }
        if ((mBeanOperationInfo instanceof OpenMBeanOperationInfo)) {
            return new OpenMBeanOperationInfoSupport(mBeanOperationInfo.getName(), description, paramsToOpenParams(array), ((OpenMBeanOperationInfo)mBeanOperationInfo).getReturnOpenType(), impact, mBeanOperationInfo.getDescriptor());
        }
        return new MBeanOperationInfo(mBeanOperationInfo.getName(), description, array, mBeanOperationInfo.getReturnType(), impact, mBeanOperationInfo.getDescriptor());
    }
    
    private static MBeanAttributeInfo customize(MBeanAttributeInfo mBeanAttributeInfo, String description) {
        if (equal(description, mBeanAttributeInfo.getDescription())) {
            return mBeanAttributeInfo;
        }
        if ((mBeanAttributeInfo instanceof OpenMBeanAttributeInfo)) {
            return new OpenMBeanAttributeInfoSupport(mBeanAttributeInfo.getName(), description, ((OpenMBeanAttributeInfo)mBeanAttributeInfo).getOpenType(), mBeanAttributeInfo.isReadable(), mBeanAttributeInfo.isWritable(), mBeanAttributeInfo.isIs(), mBeanAttributeInfo.getDescriptor());
        }
        return new MBeanAttributeInfo(mBeanAttributeInfo.getName(), mBeanAttributeInfo.getType(), description, mBeanAttributeInfo.isReadable(), mBeanAttributeInfo.isWritable(), mBeanAttributeInfo.isIs(), mBeanAttributeInfo.getDescriptor());
    }
    
    private static OpenMBeanParameterInfo[] paramsToOpenParams(MBeanParameterInfo[] array) {
        if ((array instanceof OpenMBeanParameterInfo[])) {
            return (OpenMBeanParameterInfo[])(OpenMBeanParameterInfo[])array;
        }
        OpenMBeanParameterInfoSupport[] array2 = new OpenMBeanParameterInfoSupport[array.length];
        System.arraycopy(array, 0, array2, 0, array.length);
        return array2;
    }
    
    private MBeanConstructorInfo[] getConstructors(MBeanInfo mBeanInfo, Object o) {
        MBeanConstructorInfo[] constructors = this.getConstructors(mBeanInfo.getConstructors(), o);
        if (constructors == null) {
            return null;
        }
        int length = constructors.length;
        MBeanConstructorInfo[] array = new MBeanConstructorInfo[length];
        MBeanConstructorInfo mBeanConstructorInfo;
        MBeanParameterInfo[] array2;
        for (int i = 0; i < length; array[i] = customize(mBeanConstructorInfo, this.getDescription(mBeanConstructorInfo), array2), ++i) {
            mBeanConstructorInfo = constructors[i];
            MBeanParameterInfo[] signature = mBeanConstructorInfo.getSignature();
            if (signature != null) {
                int length2 = signature.length;
                array2 = new MBeanParameterInfo[length2];
                for (int j = 0; j < length2; ++j) {
                    MBeanParameterInfo mBeanParameterInfo = signature[j];
                    array2[j] = customize(mBeanParameterInfo, this.getParameterName(mBeanConstructorInfo, mBeanParameterInfo, j), this.getDescription(mBeanConstructorInfo, mBeanParameterInfo, j));
                }
            }
            else {
                array2 = null;
            }
        }
        return array;
    }
    
    private MBeanOperationInfo[] getOperations(MBeanInfo mBeanInfo) {
        MBeanOperationInfo[] operations = mBeanInfo.getOperations();
        if (operations == null) {
            return null;
        }
        int length = operations.length;
        MBeanOperationInfo[] array = new MBeanOperationInfo[length];
        MBeanOperationInfo mBeanOperationInfo;
        MBeanParameterInfo[] array2;
        for (int i = 0; i < length; array[i] = customize(mBeanOperationInfo, this.getDescription(mBeanOperationInfo), array2, this.getImpact(mBeanOperationInfo)), ++i) {
            mBeanOperationInfo = operations[i];
            MBeanParameterInfo[] signature = mBeanOperationInfo.getSignature();
            if (signature != null) {
                int length2 = signature.length;
                array2 = new MBeanParameterInfo[length2];
                for (int j = 0; j < length2; ++j) {
                    MBeanParameterInfo mBeanParameterInfo = signature[j];
                    array2[j] = customize(mBeanParameterInfo, this.getParameterName(mBeanOperationInfo, mBeanParameterInfo, j), this.getDescription(mBeanOperationInfo, mBeanParameterInfo, j));
                }
            }
            else {
                array2 = null;
            }
        }
        return array;
    }
    
    private MBeanAttributeInfo[] getAttributes(MBeanInfo mBeanInfo) {
        MBeanAttributeInfo[] attributes = mBeanInfo.getAttributes();
        if (attributes == null) {
            return null;
        }
        int length = attributes.length;
        MBeanAttributeInfo[] array = new MBeanAttributeInfo[length];
        for (int i = 0; i < length; ++i) {
            MBeanAttributeInfo mBeanAttributeInfo = attributes[i];
            array[i] = customize(mBeanAttributeInfo, this.getDescription(mBeanAttributeInfo));
        }
        return array;
    }
    
    @Override
    public ObjectName preRegister(MBeanServer mBeanServer, ObjectName objectName) throws Exception {
        this.mbean.register(mBeanServer, objectName);
        return objectName;
    }
    
    @Override
    public void postRegister(Boolean b) {
        if (!(boolean)b) {
            this.mbean.unregister();
        }
    }
    
    @Override
    public void preDeregister() throws Exception {
    }
    
    @Override
    public void postDeregister() {
        this.mbean.unregister();
    }
    
    static boolean immutableInfo(Class<? extends StandardMBean> clazz) {
        if (clazz == StandardMBean.class || clazz == StandardEmitterMBean.class) {
            return true;
        }
        synchronized (StandardMBean.mbeanInfoSafeMap) {
            Boolean value = (Boolean)StandardMBean.mbeanInfoSafeMap.get(clazz);
            if (value == null) {
                try {
                    value = (Boolean)AccessController.doPrivileged(new StandardMBean.MBeanInfoSafeAction(clazz));
                }
                catch (Exception ex) {
                    value = Boolean.valueOf(false);
                }
                StandardMBean.mbeanInfoSafeMap.put(clazz, value);
            }
            return (boolean)value;
        }
    }
    
    static boolean overrides(Class<?> superclass, Class<?> clazz, String name, Class<?>... parameterTypes) {
        while (superclass != clazz) {
            try {
                superclass.getDeclaredMethod(name, parameterTypes);
                return true;
            }
            catch (NoSuchMethodException ex) {
                superclass = superclass.getSuperclass();
            }
        }
        return false;
    }
    
    static {
        descriptors = DescriptorCache.getInstance(JMX.proof);
        mbeanInfoSafeMap = new WeakHashMap();
    }
    
    private static class MBeanInfoSafeAction implements PrivilegedAction<Boolean>
    {
        private final Class<?> subclass;
        
        MBeanInfoSafeAction(Class<?> subclass) {
            this.subclass = subclass;
        }
        
        @Override
        public Boolean run() {
            if (StandardMBean.overrides(this.subclass, StandardMBean.class, "cacheMBeanInfo", new Class[] { MBeanInfo.class })) {
                return Boolean.valueOf(false);
            }
            if (StandardMBean.overrides(this.subclass, StandardMBean.class, "getCachedMBeanInfo", (Class<?>[])null)) {
                return Boolean.valueOf(false);
            }
            if (StandardMBean.overrides(this.subclass, StandardMBean.class, "getMBeanInfo", (Class<?>[])null)) {
                return Boolean.valueOf(false);
            }
            if (StandardEmitterMBean.class.isAssignableFrom(this.subclass) && StandardMBean.overrides(this.subclass, StandardEmitterMBean.class, "getNotificationInfo", (Class<?>[])null)) {
                return Boolean.valueOf(false);
            }
            return Boolean.valueOf(true);
        }
    }
}
